;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                        ;;
;;             Center for Spoken Language Understanding                   ;;
;;        Oregon Graduate Institute of Science & Technology               ;;
;;                         Portland, OR USA                               ;;
;;                        Copyright (c) 1999                              ;;
;;                                                                        ;;
;;      This module is not part of the CSTR/University of Edinburgh       ;;
;;               release of the Festival TTS system.                      ;;
;;                                                                        ;;
;;  In addition to any conditions disclaimers below, please see the file  ;;
;;  "license_ogi_tts.txt" distributed with this software for information  ;;
;;  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  ;;
;;                                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                       ;;
;;                Center for Spoken Language Understanding               ;;
;;                     Oregon Graduate Institute                         ;;
;;                         Copyright (c) 1998                            ;;
;;                        All Rights Reserved.                           ;;
;;                                                                       ;;
;;  Permission to use, copy, modify, distribute this software and its    ;;
;;  documentation for research, educational and individual use only, is  ;;
;;  hereby granted without fee, subject to the following conditions:     ;;
;;   1. The code must retain the above copyright notice, this list of    ;;
;;      conditions and the following disclaimer.                         ;;
;;   2. Any modifications must be clearly marked as such.                ;;
;;   3. Original authors' names are not deleted.                         ;;
;;  This software may not be used for commercial purposes without        ;;
;;  specific prior written permission from the authors.                  ;;
;;                                                                       ;;
;;  OREGON GRADUTE INSTITUTE AND THE CONTRIBUTORS TO THIS WORK           ;;
;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
;;  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ;;
;;  SHALL OREGON GRADUATE INSTITUTE NOR THE CONTRIBUTORS BE LIABLE       ;;
;;  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ;;
;;  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ;;
;;  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ;;
;;  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ;;
;;  THIS SOFTWARE.                                                       ;;
;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                       ;;
;;       A German Voice for the Festival Speech Synthesis System         ;;
;;        Authors: Karin Mueller, Bettina Saeuberlich, Horst Meyer,      ;;
;;                 Mike Macon and Alan W Black                           ;;
;;          Date:  OGI, June/July 1998                                   ;;
;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Use an extra lexicon for preprocessing removing umlaut (for Celex)
;;; Thus graman-pre is the lexicon used which simply removes umlaut
;;; then call german real
;;;

(require 'lts)           ;; system lts routines for using rules
(require 'de-celex-lts)  ;; lts rules trained from CELEX

;;; Lexicon
(lex.create "german-real")
(lex.set.phoneset "german-sampa")
(lex.set.lts.method "german-unknown-word")
(lex.set.compile.file (path-append libdir "dicts" "de-celex.out"))
(lex.add.entry
   '("Guten" nil (((g U) 0) ((t n) 0))))
(lex.add.entry
   '("Tag" nil (((t a: k) 1))))
(lex.add.entry
   '("Topf" nil (((t O pf) 1))))
(lex.add.entry
   '("der" nil (((d 6) 0))))
(lex.add.entry
   '("ist" nil (((I s t) 0))))
(lex.add.entry
   '("heiss" nil (((h aI s) 1))))
(lex.add.entry
   '("Oregon" nil (((o:) 1) ((R E) 0) ((g O n) 0) )))
(lex.add.entry
   '("Graduierten" nil (((g R a) 0) ((d U) 0) ((I R) 1) ((t E n) 0) )))
(lex.add.entry
   '("Karin" nil (((k a:) 1) ((R I n) 0))))
(lex.add.entry
   '("Bettina" nil (((b E) 0) ((t I) 1) ((n a) 0) )))
(lex.add.entry
   '("Horst" nil (((h O R s t) 1))))
(lex.add.entry
   '("Meyer" nil (((m aI) 1) ((@ R) 0))))
(lex.add.entry
   '("Mike" nil (((m aI k) 1))))
(lex.add.entry
   '("Macon" nil (((m eI) 1) ((k @ n) 0))))
 
(provide 'de-lex)

;;;
(lex.create "german-pre")
(lex.set.phoneset "german-sampa")
(lex.set.lts.method 'german-pre-lexicon)

(define (german-pre-lexicon word feats)
"(german-pre-lexicon word feats)
Preprocess the word, removing the umlauts."
  (let ((nword (lts.apply word 'DE-unumlaut)))
    (lex.select 'german-real)
    (set! ans (lex.lookup (apply string-append nword) feats))
    (lex.select 'german-pre)
    ans
    )
)

(define (german-unknown-word word feats)
"(german-unknown-word word feats)
This word is not in the lexicon, so use LTS on it."
  (let ((dcword (downcase word))
	(syls) (phones))
    (if (string-matches dcword "[a-z]*")
	(begin
	  (set! phones (lts_predict dcword de-celex_lts_cart_list))
	  (set! syls (lex.syllabify.phstress phones))
	  (set! syls (de-add_lex_stress word syls))
	  (list word nil syls))
	(begin
	  (format t "unknown word %s \n" word)
	  (lex.lookup 'unbekannt)))))

(define (de-add_lex_stress word syls)
"Very naive adding of stress, this should be improved."
  (cond
   ((> (length syls) 1)
      (set-car! (cdr (nth (- (length syls) 2) syls)) 1))
   ((word-is-content word english_guess_pos)
      (set-car! (cdr (car syls)) 1)))
  syls)


(lts.ruleset
 DE-unumlaut
 ( )
 (
  ( [ a ] = a )
  ( [ b ] = b )
  ( [ c ] = c )
  ( [ d ] = d )
  ( [ e ] = e )
  ( [ f ] = f )
  ( [ g ] = g )
  ( [ h ] = h )
  ( [ i ] = i )
  ( [ j ] = j )
  ( [ k ] = k ) 
  ( [ l ] = l )
  ( [ m ] = m )
  ( [ n ] = n )
  ( [ o ] = o )
  ( [ p ] = p )
  ( [ q ] = q )
  ( [ r ] = r )
  ( [ s ] = s )
  ( [ t ] = t )
  ( [ u ] = u )
  ( [ v ] = v ) 
  ( [ w ] = w )
  ( [ x ] = x )
  ( [ y ] = y )
  ( [ z ] = z )
  ( [  ] = a e )
  ( [  ] = o e ) 
  ( [  ] = u e )
  ( [  ] = s s )
  ( [ "\"a" ] = a e )
  ( [ "\"o" ] = o e ) 
  ( [ "\"u" ] = u e )
  ( [ "\"s" ] = s s )
  ( [ "\\\"a" ] = a e )
  ( [ "\\\"o" ] = o e ) 
  ( [ "\\\"u" ] = u e )
  ( [ "\\\"s" ] = s s )
  ( [ A ] = A )
  ( [ B ] = B )
  ( [ C ] = C )
  ( [ D ] = D )
  ( [ E ] = E )
  ( [ F ] = F )
  ( [ G ] = G ) 
  ( [ H ] = H )
  ( [ I ] = I )
  ( [ J ] = J )
  ( [ K ] = K )
  ( [ L ] = L )
  ( [ M ] = M )
  ( [ N ] = N )
  ( [ O ] = O )
  ( [ P ] = P )
  ( [ Q ] = Q )
  ( [ R ] = R ) 
  ( [ S ] = S )
  ( [ T ] = T )
  ( [ U ] = U )
  ( [ V ] = V )
  ( [ W ] = W )
  ( [ X ] = X )
  ( [ Y ] = Y )
  ( [ Z ] = Z )
  ( [ "\"A" ] = A e )
  ( [ "\"O" ] = O e )
  ( [ "\"U" ] = U e ) 
  ( [ "\\\"A" ] = A e )
  ( [ "\\\"O" ] = O e )
  ( [ "\\\"U" ] = U e ) 
  ( [  ] = A e ) 
  ( [  ] = O e ) 
  ( [  ] = U e ) 
  ( [  ] = A e ) 
  ( [  ] = O e ) 
  ( [  ] = U e ) 
  ( [  ] = a )
  ( [  ] = e )
  ( [  ] = i )
  ( [  ] = o )
  ( [  ] = u )
  ( [ "" ] = s )
  ( [  ] = s )
  ( [  ] =  n j )
  ( [ "\'" ] = "\'" )
  ( [ : ] = : )
  ( [ ~ ] = ~ )
  ( [ "\"" ] = "\"" )
  ( [  ] = A )
  ( [  ] = E )
  ( [  ] = I )
  ( [  ] = O )
  ( [  ] = U )
  ( [ "" ] = S )
  ( [  ] =  N j )
))
