;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                        ;;
;;             Center for Spoken Language Understanding                   ;;
;;        Oregon Graduate Institute of Science & Technology               ;;
;;                         Portland, OR USA                               ;;
;;                        Copyright (c) 1999                              ;;
;;                                                                        ;;
;;      This module is not part of the CSTR/University of Edinburgh       ;;
;;               release of the Festival TTS system.                      ;;
;;                                                                        ;;
;;  In addition to any conditions disclaimers below, please see the file  ;;
;;  "license_ogi_tts.txt" distributed with this software for information  ;;
;;  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  ;;
;;                                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                       ;;
;;                Center for Spoken Language Understanding               ;;
;;                     Oregon Graduate Institute                         ;;
;;                         Copyright (c) 1998                            ;;
;;                        All Rights Reserved.                           ;;
;;                                                                       ;;
;;  Permission to use, copy, modify, distribute this software and its    ;;
;;  documentation for research, educational and individual use only, is  ;;
;;  hereby granted without fee, subject to the following conditions:     ;;
;;   1. The code must retain the above copyright notice, this list of    ;;
;;      conditions and the following disclaimer.                         ;;
;;   2. Any modifications must be clearly marked as such.                ;;
;;   3. Original authors' names are not deleted.                         ;;
;;  This software may not be used for commercial purposes without        ;;
;;  specific prior written permission from the authors.                  ;;
;;                                                                       ;;
;;  OREGON GRADUTE INSTITUTE AND THE CONTRIBUTORS TO THIS WORK           ;;
;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
;;  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ;;
;;  SHALL OREGON GRADUATE INSTITUTE NOR THE CONTRIBUTORS BE LIABLE       ;;
;;  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ;;
;;  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ;;
;;  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ;;
;;  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ;;
;;  THIS SOFTWARE.                                                       ;;
;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                       ;;
;;       A German Voice for the Festival Speech Synthesis System         ;;
;;        Authors: Karin Mueller, Bettina Saeuberlich, Horst Meyer,      ;;
;;                 Mike Macon and Alan W Black                           ;;
;;          Date:  OGI, June/July 1998                                   ;;
;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  Tokenization rules for german
;;;
;;;  Particularly numbers and symbols.
;;; 
    
(define (german_number name)
"(german_number name)
Convert a string of digits into a list of words saying the number."
  (if (string-matches name "0")
      (list "null")
      (german_number_from_digits (symbolexplode name))))

(define (just_zeros digits)
"(just_zeros digits)
If this only contains 0s then we just do something different."
 (cond
  ((not digits) t)
  ((string-equal "0" (car digits))
   (just_zeros (cdr digits)))
  (t nil)))

(define (german_number_from_digits digits)
  "(german_number_from_digits digits)
Takes a list of digits and converts it to a list of words
saying the number."
  (let ((l (length digits)))
    (cond
     ((equal? l 0)
      nil)
     ((string-equal (car digits) "0")
      (german_number_from_digits (cdr digits)))
     ((equal? l 1);; single digit
      (cond 
       ((string-equal (car digits) "0") (list "null"))
       ((string-equal (car digits) "1") (list "eins"))
       ((string-equal (car digits) "2") (list "zwei"))
       ((string-equal (car digits) "3") (list "drei"))
       ((string-equal (car digits) "4") (list "vier"))
       ((string-equal (car digits) "5") (list "fnf"))
       ((string-equal (car digits) "6") (list "sechs"))
       ((string-equal (car digits) "7") (list "sieben"))
       ((string-equal (car digits) "8") (list "acht"))
       ((string-equal (car digits) "9") (list "neun"))
       ;; fill in the rest
       (t (list "equis"))));; $$$ what should say?
     ((equal? l 2);; less than 100
      (cond
       ((string-equal (car digits) "0");; 0x
	(german_number_from_digits (car digits)))
     
       ((string-equal (car digits) "1");; 1x
	(cond
	 ((string-equal (car (cdr digits)) "0") (list "zehn"))
	 ((string-equal (car (cdr digits)) "1") (list "elf"))
	 ((string-equal (car (cdr digits)) "2") (list "zwlf"))
	 (t 
	  (list (car (german_number_from_digits (cdr digits))) "zehn" ))))
     
       ((string-equal (car digits) "2");; 2x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "zwanzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "zwanzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "zwanzig" ))))

       ((string-equal (car digits) "3");; 3x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "dreiig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "dreiig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "dreiig" ))))

       ((string-equal (car digits) "4");; 4x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "vierzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "vierzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "vierzig" ))))

       ((string-equal (car digits) "5");; 5x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "fnfzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "fnfzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "fnfzig" ))))

       ((string-equal (car digits) "6");; 6x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "sechzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "sechzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "sechzig" ))))

       ((string-equal (car digits) "7");; 7x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "siebzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "siebzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "siebzig" ))))

       ((string-equal (car digits) "8");; 8x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "achtzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "achtzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "achtzig" ))))

       ((string-equal (car digits) "9");; 9x
        (cond
	 ((string-equal (car (cdr digits)) "0") (list "neunzig"))
	 ((string-equal (car (cdr digits)) "1") (list "ein" "und" "neunzig"))
         (t                 
	    (list (car (german_number_from_digits (cdr digits))) "und" "neunzig" ))))
       ))

     ((equal? l 3);; in the hundreds
      (cond 
        ((string-equal (car digits) "1");; 1xx
	(if (just_zeros (cdr digits)) (list "hundert")
	    (cons "einhundert" (german_number_from_digits (cdr digits)))))

       (t;; ?xx
	(append (german_number_from_digits (list (car digits))) 
		(list "hundert") 
		(german_number_from_digits (cdr digits))))
       ))

     ((< l 7)
      (let ((sub_thousands 
	     (list 
	      (car (cdr (cdr (reverse digits))))
	      (car (cdr (reverse digits)))
	      (car (reverse digits))))
	    (thousands (reverse (cdr (cdr (cdr (reverse digits)))))))
	(set! x (german_number_from_digits thousands))
	(append
	 (if (string-equal (car x) "ein") nil x)
	 (list "tausend")
	 (german_number_from_digits sub_thousands))))

     ((< l 13)
      (let ((sub_million 
	     (list 
	      (car (cdr (cdr (cdr (cdr (cdr(reverse digits)))))))
	      (car (cdr (cdr (cdr (cdr (reverse digits))))))
	      (car (cdr (cdr (cdr (reverse digits)))))
	      (car (cdr (cdr (reverse digits))))
	      (car (cdr (reverse digits)))
	      (car (reverse digits))
	      ))
	    (millions (reverse (cdr (cdr (cdr (cdr (cdr (cdr (reverse digits))))))))))
	(set! x (german_number_from_digits millions))
	(append
	 (if (string-equal (car x) "eine") 
	     (list "eine" "millon") 
	     (append x (list "millionen")))
	 (german_number_from_digits sub_million))))

     (t
      (list "eine" "seeeehr" "grosse" "zahl")))))

(define (german_token_to_words token name)
  "(german_token_to_words TOKEN NAME)
Returns a list of words for the NAME from TOKEN.  This primarily
allows the treatment of numbers, money etc."
  (cond
   ((string-matches name "[0-9]+")
    (german_number name))
   ((string-matches name ".*-.*")
    (list (german_token_to_words (string-before name "-")) (german_token_to_words (string-after name "-"))))
   ;; dates, decimals, money etc
   (t
    (list name))))

(provide 'dtoken)

